/*
 * Ultra Wide Band
 * Miscellaneous helpers
 *
 * Copyright (C) 2007 Intel Corporation
 * Inaky Perez-Gonzalez <inaky.perez-gonzalez@intel.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef __LINUX__UWB__UTIL_H__
#define __LINUX__UWB__UTIL_H__

#include <asm/byteorder.h>

/*
 * Declare Little Endian bitfields
 *
 * These macros are used to switch the order of the bitfields when
 * packing with Little Endian in mind (ie: for mapping to a hw type
 * that you have to send or read from a device).
 *
 * /me asks for __attribute__((endian(...))) in GCC
 *
 * WARNING! When using multibyte bitfields, you need to convert the
 *          data from Little Endian to CPU before you can access the
 *          bitfield (to make it simpler):
 *
 *          union something {
 *                  le16 value;
 *                  DECL_BF_LE3(
 *                          u16  bf0:3,
 *                          u16  bf1:10,
 *                          u16  bf2:3
 *                  ) __attribute__((packed));
 *          };
 *
 *          ...
 *
 *          union something s;
 *
 *          s.value = le16_to_cpu(something LE read from hw);
 *          frame_count = s.bf1;
 *
 */
#ifdef __LITTLE_ENDIAN_BITFIELD
#define DECL_BF_LE2(a, b)                   struct { a; b; }
#define DECL_BF_LE3(a, b, c)                struct { a; b; c; }
#define DECL_BF_LE4(a, b, c, d)             struct { a; b; c; d ; }
#define DECL_BF_LE5(a, b, c, d, e)          struct { a; b; c; d; e; }
#define DECL_BF_LE6(a, b, c, d, e, f)       struct { a; b; c; d; e; f; }
#define DECL_BF_LE7(a, b, c, d, e, f, g)    struct { a; b; c; d; e; f; g; }
#define DECL_BF_LE8(a, b, c, d, e, f, g, h) struct { a; b; c; d; e; f; g; h; }
#define DECL_BF_LE12(a, b, c, d, e, f, g, h, i, j, k, l) \
	struct { a; b; c; d; e; f; g; h; i; j; k; l; }

#elif __BIG_ENDIAN_BITFIELD

#define DECL_BF_LE2(a, b)                   struct { b; a; }
#define DECL_BF_LE3(a, b, c)                struct { c; b; a; }
#define DECL_BF_LE4(a, b, c, d)             struct { d; c; b; a; }
#define DECL_BF_LE5(a, b, c, d, e)          struct { e; d; c; b; a; }
#define DECL_BF_LE6(a, b, c, d, e, f)       struct { f; e; d; c; b; a; }
#define DECL_BF_LE7(a, b, c, d, e, f, g)    struct { g; f; e; d; c; b; a; }
#define DECL_BF_LE8(a, b, c, d, e, f, g, h) struct { h; g; f; e; d; c; b; a; }
#define DECL_BF_LE12(a, b, c, d, e, f, g, h, i, j, k, l) \
	struct { l; k; j; i; h; g; f; e; d; c; b; a; }

#else

#error Unknown endianess for Little Endian bitfield definition

#endif

#endif /* #ifndef __LINUX__UWB__UTIL_H__ */
